'use strict';

app.controller('LibraryLocalCtrl', ['$scope', 'LibraryService', 'RestService', 'FormService', 'FormDataService', 'BrowseFoldersService', 'formData', 'users',
	function ($scope, LibraryService, RestService, FormService, FormDataService, BrowseFoldersService, formData, users) {
	
	$scope.formData = formData;
    $scope.users = users;

	var cleanAddForm = {
        available: true,
        folderPath: '',
        userIds: [],
        supportedFileTypes: [],
        descriptiveMetadataSupported: false,
        usePoller: false,
        aliasName: '',
        ignoredFolderNames: ['@eadir','.@__thumb'],
        ignoredFileExtensions: []
    };

	$scope.addResourceOpen = false;
	$scope.newSharedFolder = angular.copy(cleanAddForm);
    $scope.editingSharedFolderIndex = null;
	var originalSharedFolders = angular.copy($scope.formData.sharedFolders);

	//todo enable poller per folder

    function resetAddForm() { 
        $scope.editingSharedFolderIndex = null;
        $scope.newSharedFolder = angular.copy(cleanAddForm);
        $scope.$broadcast("resettableFormReset"); // mark the inner form as pristine
    }

    function closeAddResourceForm() {
        $scope.addResourceOpen = false;
    }

    $scope.$on("formStateReset", function (event, modified) {
        $scope.formData['sharedFolders'] = angular.copy(originalSharedFolders);
        LibraryService.markAsPristine($scope, null);
        resetAddForm();          
    });

    $scope.sortableOptions = {
        containment: '#foldersTableContainer',
        containerPositioning: 'relative',
        orderChanged : function(obj) {
            var form = angular.element("form").scope().Form;
            LibraryService.markAsModified($scope, form);
        }
    };

    $scope.submit = function (form) {
        FormService.submitForm('repository', $scope.formData, form, $scope, function(response) {
            FormDataService.loadFormData('repository').then(function(json) {
                $scope.formData['sharedFolders'] = json.sharedFolders;
                originalSharedFolders = angular.copy($scope.formData.sharedFolders);

                LibraryService.markAsPristine($scope, form);
            });    
        });
    };

    $scope.openAddResourceForm = function() {
        $scope.newSharedFolder = prepareNewFolderModel(angular.copy(cleanAddForm));
        $scope.addResourceOpen = true;
    };

    $scope.openEditResourceForm = function(index) {
        $scope.newSharedFolder = prepareNewFolderModel(angular.copy($scope.formData.sharedFolders[index]));
        $scope.addResourceOpen = true;
        $scope.editingSharedFolderIndex = index;
    };

    $scope.cancelAddSharedFolder = function() {
        closeAddResourceForm();
        resetAddForm();
    };

    $scope.addSharedFolder = function(form, index) {
        var valid = FormService.validateForm(form);
        if(valid) {
            var newSource = normalizeNewFolderModel(angular.copy($scope.newSharedFolder));
            if(index != null) {
                // edit
                $scope.formData.sharedFolders[index] = newSource;
            } else {
                // new
                $scope.formData.sharedFolders.push(newSource);
            }

            LibraryService.markAsModified($scope, form.$$parentForm);
            closeAddResourceForm();
            resetAddForm();
        }     
    };

    $scope.removeSharedFolder = function(index, form) {
        var repos = $scope.formData.sharedFolders;
        repos.splice(index, 1);
        LibraryService.markAsModified($scope, form);
    };

    $scope.forceRefreshLibrary = function() {
        $scope.libraryForceRefreshInProgress = true;
        RestService.invokeAction('forceLibraryRefresh', null, true, function(response) {            
            $scope.libraryForceRefreshInProgress = false;            
        }, function(response) {            
            $scope.libraryForceRefreshInProgress = false;
        })
    };

	$scope.moveResourceUp = function(index, form) {
        if(index > 0) {
            var list = $scope.formData.sharedFolders;
            list.splice(index - 1, 0, list.splice(index, 1)[0]);
            LibraryService.markAsModified($scope, form);
        }
    };

    $scope.moveResourceDown = function(index, form) {
        var list = $scope.formData.sharedFolders;
        if(index < list.length) {            
            list.splice(index + 1, 0, list.splice(index, 1)[0]);
            LibraryService.markAsModified($scope, form);
        }
    };

    $scope.videoMediaTypeToggled = function() {
    	if($scope.newSharedFolder.supportedFileTypes['VIDEO']) {
    		$scope.newSharedFolder.descriptiveMetadataSupported = true;
    	} else {
    		$scope.newSharedFolder.descriptiveMetadataSupported = false;
    	}
    };

    $scope.someMediaTypeSelected = function() {
    	var someSelected = false;
    	var typesSelection = $scope.newSharedFolder.supportedFileTypes;
    	var supportedTypes = Object.keys(typesSelection);
    	for(var i = 0; i < supportedTypes.length; i++) {
    		if(typesSelection[supportedTypes[i]] == true) {
    			someSelected = true;
    		}
    	}
    	return someSelected;
    };

    $scope.openFolderSelect = function() {
       
        var browseResult = BrowseFoldersService.openDialog('tab_folders_browse_for_folder');

        browseResult.result.then(function (selectedFolder) {
            $scope.newSharedFolder.folderPath = selectedFolder; 
        });
    };

    function prepareNewFolderModel(model) {
    	var updatedModel = LibraryService.prepareUserIds(model);

    	var transformedMediaTypes = {};
        for(var i = 0; i < updatedModel.supportedFileTypes.length; i++) {
            transformedMediaTypes[updatedModel.supportedFileTypes[i]] = true;
        }
        updatedModel.supportedFileTypes = transformedMediaTypes;
        updatedModel.ignoredFolderNamesCsv = updatedModel.ignoredFolderNames.join(',');
        updatedModel.ignoredFileExtensionsCsv = updatedModel.ignoredFileExtensions.join(',');
    	return updatedModel;
    }

    function normalizeNewFolderModel(model) {
		var updatedModel = LibraryService.normalizeUserIds(model);

	 	var fts = Object.keys(model.supportedFileTypes);
        var selectedFts = [];
        var selectedNumber = 0;
        for(var i = 0; i < fts.length; i++) {
            if(updatedModel.supportedFileTypes[fts[i]] == true) {
                selectedFts[selectedNumber] = fts[i];
                selectedNumber ++;
            }
        }
        updatedModel.supportedFileTypes = selectedFts;

        if(!updatedModel.ignoredFolderNamesCsv || !updatedModel.ignoredFolderNamesCsv.trim()) {
            updatedModel.ignoredFolderNames = [];
        } else {
            updatedModel.ignoredFolderNames = updatedModel.ignoredFolderNamesCsv.split(',');
        }
        if(!updatedModel.ignoredFileExtensionsCsv || !updatedModel.ignoredFileExtensionsCsv.trim()) {
            updatedModel.ignoredFileExtensions = [];
        } else {
            updatedModel.ignoredFileExtensions = updatedModel.ignoredFileExtensionsCsv.split(',');
        }
    	return updatedModel;
    }
}]);